#uses "fwWebMonitor/fwWebMonitorConsts.ctl"


/** 
  Creates and prepares datapoint for new monitored site with default _dp_fct config
  
  @return name of newly created datapoint
*/
string fwWebMonitor_createSiteEntry() {
  string siteEntryDpName = fwWebMonitor_getNextSiteEntryDpName();
  dpCreate(siteEntryName, FWWM_MONITORED_ENTRY_TYPE_NAME);
  
  dyn_string dpFctParams = makeDynString("_Connections.Ctrl.ManNums:_original.._value", //p1
                                    FWWM_SYSTEM_DP_NAME + ".ctrlManagerNumber:_original.._value", //p2
                                    siteEntryDpName + ".stateRaw:_original.._value"); //p3

  dpSet(siteEntryDpName+".state:_dp_fct.._type", 60,
        siteEntryDpName+".state:_dp_fct.._param", dpFctParams,
        siteEntryDpName+".state:_dp_fct.._fct", "dynContains(p1, p2) ? p3 : -1");

  return siteEntryDpName;
}

/** 
  Generates a new site entry datapoint name using internal sequence
  
  @return datapoint name for site entry
*/
string fwWebMonitor_getNextSiteEntryDpName() {
  uint sequence;
  dpGet(FWWM_SYSTEM_DP_NAME + ".siteSequenceNumber", sequence);
  sequence++;
  dpSet(FWWM_SYSTEM_DP_NAME + ".siteSequenceNumber", sequence);

  return FWWM_MONITORED_ENTRY_DP_PREFIX + FWWM_MONITORED_ENTRY_DP_NAME + sequence;  
}


/** 
  Saves site information in datapoint 
  
  @param dpName              input, target datapoint to saved values in
  @param siteName            input, site name
  @param url                 input, site URL
  @param followRedirection   input, true value enables to follow HTTP redirections for this site
  @param redirectionLimit    input, number of HTTP redirections to be followed
  @param expectedStatusCode  input, expected HTTP status code
  @param enabled             input, true value enables site monitoring
  @param statusDescription   input, site status description text
*/
void fwWebMonitor_setSiteEntryValues(string dpName, string siteName, string url, bool followRedirection, uint redirectionLimit, int expectedStatusCode, bool enabled, string statusDescription = "New entry") {
  if(!fwWebMonitor_checkNonEmptyString(dpName)) {
    fwWebMonitor_throw("fwWebMonitor_setSiteEntryValues() - empty DP name", "ERROR", 5);
    return;
  }

  if(!fwWebMonitor_checkNonEmptyString(siteName)) {
    fwWebMonitor_throw("fwWebMonitor_setSiteEntryValues() - empty site name", "ERROR", 5);
    return;
  }
  
  if(!fwWebMonitor_checkNonEmptyString(url)) {
    fwWebMonitor_throw("fwWebMonitor_setSiteEntryValues() - empty URL", "ERROR", 5);
    return;
  }
  
  string alias = siteName;
  strreplace(alias, " ", "/");
  
  dpSet(dpName+".name", siteName, 
        dpName+".url", url, 
        dpName+".followRedirection", followRedirection, 
        dpName+".redirectionLimit", redirectionLimit,
        dpName+".expectedStatusCode", expectedStatusCode,
        dpName+".enabled", true,
        dpName+".statusText", statusDescription,
        dpName+".lastCheck", getCurrentTime(),
        dpName+".stateRaw", FWWM_MONITORED_ENTRY_STATE_ERROR,
        dpName+".dpName", dpName);
  
  dpSetAlias(dpName+".", alias);
}

/** 
  Checks if the input string is empty
  
  @param input, value to be checked

  @return true if the value is not emptry, false otherwise
*/
bool fwWebMonitor_checkNonEmptyString(string value) {
  bool result = false;
  if (strlen(value)>0)
    result = true;
  
  return result;
}

/** 
  Prepares and throw error object for given message, severity and code
  
  @param msg       error message
  @param severity  error severity
  @param code      error code
*/
void fwWebMonitor_throw(string message, string severity = "ERROR", int code = 1)
{
  int priority = PRIO_WARNING; 
  int type = ERR_CONTROL;
  
  switch(strtoupper(severity))
  {
    case "INFO": priority = PRIO_INFO; code = 10; break;
    case "WARNING": priority = PRIO_WARNING; break;
    case "ERROR": priority = PRIO_SEVERE; break;
  }
  
  errClass err = makeError("fwWebMonitor", priority, type, code, message);
  throwError(err);
  
  return;
}

/** 
  Gives datapoint names for all monitored sites
  
  @return list of datapoint names
*/
dyn_string fwWebMonitor_getSiteEntryNames() {
  dyn_string result;
  string query = "SELECT '_original.._value' FROM '*.***' WHERE _EL = \"dpName\" AND  _DPT = \"" + FWWM_MONITORED_ENTRY_TYPE_NAME + "\" AND  _DP LIKE \"" + FWWM_MONITORED_ENTRY_DP_PREFIX + "*\"";
  dyn_dyn_anytype siteEntries;
  dpQuery(query, siteEntries);

  for (int i = 2; i <= dynlen(siteEntries); i++) {
    dynAppend(result, siteEntries[i][2]);
  }
  return result;
//   //dpNames returns different order than dpQuery which keeps the order of datapoints in db
//   return dpNames(FWWM_MONITORED_ENTRY_DP_PREFIX+"*",FWWM_MONITORED_ENTRY_TYPE_NAME);
}

/** 
  Reads site information for given datapoint name
  
  @param siteEntryDpName  datapoint name to read information from 
  
  @return map with site information
*/
mapping fwWebMonitor_getSiteEntryData(string siteEntryDpName) {
  string name, url, statusText, dpName;
  bool enabled, followRedirection;
  time lastCheck;
  int lastStatus, expectedStatusCode, state;
  uint redirectionLimit;
  
  dpGet(siteEntryDpName+".name", name,
        siteEntryDpName+".url", url,
        siteEntryDpName+".lastStatus", lastStatus,
        siteEntryDpName+".expectedStatusCode", expectedStatusCode,
        siteEntryDpName+".statusText", statusText,
        siteEntryDpName+".enabled", enabled,
        siteEntryDpName+".lastCheck", lastCheck,
        siteEntryDpName+".followRedirection", followRedirection,
        siteEntryDpName+".redirectionLimit", redirectionLimit,
        siteEntryDpName+".state", state,
        siteEntryDpName+".dpName", dpName);
  
  mapping result;
  result["Name"] = name;
  result["URL"] = url;
  result["LastStatus"] = lastStatus;
  result["ExpectedStatus"] = expectedStatusCode;
  result["StatusPhrase"] = statusText;
  result["Enabled"] = enabled;
  result["LastCheck"] = lastCheck;
  result["FollowRedirection"] = followRedirection;
  result["RedirectionLimit"] = redirectionLimit;
  result["Ok"] = (state == FWWM_MONITORED_ENTRY_STATE_OK);
  result["dpName"] = dpName;
  return result;
}

/** 
  Reads fwWebMonitor system setting
  
  @return map with system setting
*/
mapping fwWebMonitor_getSystemSetting() {
  uint siteSequenceNumber, siteCheckInterval, tableRefreshInterval, ctrlManagerNumber;
  
  dpGet(FWWM_SYSTEM_DP_NAME+".siteSequenceNumber", siteSequenceNumber,
        FWWM_SYSTEM_DP_NAME+".siteCheckInterval", siteCheckInterval,
        FWWM_SYSTEM_DP_NAME+".tableRefreshInterval", tableRefreshInterval,
        FWWM_SYSTEM_DP_NAME+".ctrlManagerNumber", ctrlManagerNumber);
  
  mapping result;
  result["siteSequenceNumber"] = siteSequenceNumber;
  result["siteCheckInterval"] = siteCheckInterval;
  result["tableRefreshInterval"] = tableRefreshInterval;
  result["ctrlManagerNumber"] = ctrlManagerNumber;
  return result;
}

/** 
  Writes fwWebMonitor system setting
  
  @param siteSequenceNumber   current sequence value
  @param siteCheckInterval    
  @param tableRefreshInterval 
  @param ctrlManagerNumber 
*/
void fwWebMonitor_setSystemSetting(uint siteSequenceNumber, uint siteCheckInterval, uint tableRefreshInterval, uint ctrlManagerNumber) {
  if(siteCheckInterval < 1) {
    fwWebMonitor_throw("fwWebMonitor_setSystemSetting() - siteCheckInterval has to be greater than 0", "ERROR", 5);
    return;
  }

  if(tableRefreshInterval < 1) {
    fwWebMonitor_throw("fwWebMonitor_setSystemSetting() - tableRefreshInterval has to be greater than 0", "ERROR", 5);
    return;
  }
  
  dpSet(FWWM_SYSTEM_DP_NAME+".siteSequenceNumber", siteSequenceNumber,
        FWWM_SYSTEM_DP_NAME+".siteCheckInterval", siteCheckInterval,
        FWWM_SYSTEM_DP_NAME+".tableRefreshInterval", tableRefreshInterval,
        FWWM_SYSTEM_DP_NAME+".ctrlManagerNumber", ctrlManagerNumber);
}

/** 
  Enables/Disables site monitoring
  
  @param siteEntryDpName  site datapoint name
  @param enabled          true value enables monitoring, false disables it
*/
void fwWebMonitor_setSiteEnabled(string siteEntryDpName, bool enabled) {
  dpSet(siteEntryDpName+".enabled", enabled);
}

/** 
  Enables/Disables HTTP redirections given site
  
  @param siteEntryDpName    site datapoint name
  @param followRedirection  true value enables following of HTTP redirection, false disables it
*/
void fwWebMonitor_setSiteFollowRedirection(string siteEntryDpName, bool followRedirection) {
  dpSet(siteEntryDpName+".followRedirection", followRedirection);
}

/** 
  Sets the number of HTTP redirections to be followed for given site
  
  @param siteEntryName     site datapoint name
  @param redirectionLimit  number of allowed redirections
*/
void fwWebMonitor_setSiteRedirectionLimit(string siteEntryName, uint redirectionLimit) {
  dpSet(siteEntryName+".redirectionLimit", redirectionLimit);
}

/** 
  Reads the table refresh interval
  
  @return  table refresh interval in seconds
*/
uint fwWebMonitor_getTableRefreshInterval() {
  uint interval = 0;
  dpSet(FWWM_SYSTEM_DP_NAME+".tableRefreshInterval", interval);
  if (interval < 1)
    interval = 1;
  
  return interval;
}
